/* Implementation of the curve view.
 *
 * Copyright (c) 2003 Ole Laursen.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA.
 */

#include <config.h>

#include <algorithm>		// for max/min[_element]()
#include <deque>

#include <libgnomecanvasmm/line.h>
#include <libgnomecanvasmm/point.h>
#include <gconfmm/client.h>

#include "ucompose.hpp"

#include "curve-view.hpp"
#include "applet.hpp"
#include "monitor.hpp"

#include "i18n.hpp"


//
// class Curve - represents a line curve
//

class Curve
{
public:
  Curve(Monitor *monitor, const Glib::ustring &mon_dir);

  void sync_with_monitor(unsigned int max_samples); // gather info from monitor
  void draw(Gnome::Canvas::Canvas &canvas, // redraw curve on canvas
	    const Glib::RefPtr<Gnome::Conf::Client> &client,
	    int width, int height, int samples); 

  Monitor *get_monitor();
  Glib::ustring get_tip();
  
private:
  Monitor *monitor;
  std::auto_ptr<Gnome::Canvas::Line> line;
  
  // we need to store the past values
  typedef std::deque<double> value_sequence;
  typedef value_sequence::iterator value_iterator;
  value_sequence values;

  Glib::ustring monitor_dir;
};

Curve::Curve(Monitor *m, const Glib::ustring &mon_dir)
  : monitor(m), monitor_dir(mon_dir)
{}

void Curve::sync_with_monitor(unsigned int max_samples)
{
  double val = monitor->value();
  values.push_front(val);
  
  // get rid of extra samples (there may be more than one if user changes
  // configuration)
  while (values.size() > max_samples)
    values.pop_back();
}

void Curve::draw(Gnome::Canvas::Canvas &canvas,
		 const Glib::RefPtr<Gnome::Conf::Client> &client,
		 int width, int height, int samples)
{
  double max = std::max(monitor->max(), 0.000001); // avoid division by zero
  
  value_iterator i = values.begin(), end = values.end();
  assert(i != end);	// we need at least one point

  // however, only one point is pointless
  if (std::distance(i, end) < 2) 
    return;

  // make sure line is initialised
  if (line.get() == 0) {
    line.reset(new Gnome::Canvas::Line(*canvas.root()));
    line->property_smooth() = true;
  }

  // get drawing attributes
  unsigned int color;
  double line_width;

  Gnome::Conf::Value v;
  
  v = client->get(monitor_dir + "/color");
  if (v.get_type() == Gnome::Conf::VALUE_INT)
    color = v.get_int();
  else {
    color = 0x0000FF7F;		// FIXME: use changing colors
    client->set(monitor_dir + "/color", int(color));
  }
  

  v = client->get(monitor_dir + "/line_width");
  if (v.get_type() == Gnome::Conf::VALUE_FLOAT)
    line_width = v.get_float();
  else {
    line_width = 1.2;		// FIXME: use schema?
    client->set(monitor_dir + "/line_width", line_width);
  }
  

  line->property_fill_color_rgba() = color;
  line->property_width_units() = line_width;
  

  Gnome::Canvas::Points points;
  points.reserve(values.size());

  // one less than no. of samples because two points are end-points
  double pixels_per_sample = double(width) / (samples - 1);
  
  // start from right (FIXME: reverse for some locales?)
  double x = width;
	
  do {
    double y = (1 - (*i / max)) * height;

    points.push_back(Gnome::Art::Point(x, y));
    x -= pixels_per_sample;
  } while (++i != end);

  line->property_points() = points;
}

Monitor *Curve::get_monitor()
{
  return monitor;
}

Glib::ustring Curve::get_tip()
{
  // note to translators: %1 is the name of a monitor, e.g. "CPU 1", and %2 is
  // the current measurment, e.g. "78%"
  return String::ucompose(_("%1: %2"), monitor->get_short_name(),
			  monitor->format_value(values.front()));
}


//
// class CurveView
//

CurveView::CurveView()
{
}

CurveView::~CurveView()
{
  for (curve_iterator i = curves.begin(), end = curves.end(); i != end; ++i)
    delete *i;
}


void CurveView::do_display(Applet &a)
{
  applet = &a;
  
  // canvas creation magic
  canvas.reset(new Gnome::Canvas::Canvas);
  canvas->property_aa() = true;
  a.get_container().add(*canvas);
}

void CurveView::do_update()
{
  // first update the configured attributes
  Glib::RefPtr<Gnome::Conf::Client> &client = applet->get_gconf_client();
  Glib::ustring dir = applet->get_gconf_dir();

  // FIXME: use schemas?
  if (client->get(dir + "/viewer/size").get_type() == Gnome::Conf::VALUE_INT)
    size = client->get_int(dir + "/viewer/size");
  else {
    size = 60;
    client->set(dir + "/viewer/size", size);
  }

  if (client->get(dir + "/viewer/samples").get_type() == Gnome::Conf::VALUE_INT)
    samples = client->get_int(dir + "/viewer/samples");
  else {
    samples = 30;
    client->set(dir + "/viewer/samples", int(samples));
  }
  
  // ensure the canvas is shown
  resize_canvas();
  canvas->show();

  // then loop through each curve
  Glib::ustring tip;
	
  for (curve_iterator i = curves.begin(), end = curves.end(); i != end; ++i) {
    Curve &curve = **i;

    curve.sync_with_monitor(samples);

    if (tip.empty())
      tip = curve.get_tip();
    else
      // note to translators: this is used for composing a list of monitors; %1
      // is the previous part of the list and %2 is the part to append
      tip = String::ucompose(_("%1\n%2"), tip, curve.get_tip());

    curve.draw(*canvas, client, width(), height(), samples);
  }
  
  tooltips.set_tip(*canvas, tip);
}

void CurveView::do_attach(Monitor *monitor, const Glib::ustring &mon_dir)
{
  curves.push_back(new Curve(monitor, mon_dir));
}

void CurveView::do_detach(Monitor *monitor)
{
  for (curve_iterator i = curves.begin(), end = curves.end(); i != end; ++i)
    if ((*i)->get_monitor() == monitor) {
      delete *i;
      curves.erase(i);
      return;
    }

  g_assert_not_reached();
}

int CurveView::width() const
{
  if (applet->horizontal())
    return size;
  else
    return applet->get_size();
}

int CurveView::height() const
{
  if (applet->horizontal())
    return applet->get_size();
  else
    return size;
}

void CurveView::resize_canvas()
{
  int w = width(), h = height();
  
  canvas->set_scroll_region(0, 0, w, h);
  canvas->set_size_request(w, h);
}
